// BoundingBox.cpp: implementation of the CBoundingBox class.
//
//////////////////////////////////////////////////////////////////////

#include "BoundingBox.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBoundingBox::CBoundingBox()
{
	m_bVisible = true;
	m_iRenderMode = GL_LINE_LOOP;
	m_pMdlMatrix = NULL;
	m_Color = RGBA(1,0,0,0);
}

CBoundingBox::~CBoundingBox()
{

}

void CBoundingBox::Calculate(CVector min, CVector max)
{
	m_Min = min;
	m_Max = max;

	if(m_pMdlMatrix !=NULL)
	{
		m_Min+= CVector(m_pMdlMatrix->m_matrix[12], m_pMdlMatrix->m_matrix[13], m_pMdlMatrix->m_matrix[14]);
		m_Max+= CVector(m_pMdlMatrix->m_matrix[12], m_pMdlMatrix->m_matrix[13], m_pMdlMatrix->m_matrix[14]);;
	}

	// Front side - at y = min.y
	m_Vert[0] = min;
	m_Vert[1] = CVector(max.x, min.y, min.z);
	m_Vert[2] = CVector(max.x, min.y, max.z);
	m_Vert[3] = CVector(min.x, min.y, max.z);

	// Back side - at y = max.y
	m_Vert[4] = CVector(min.x, max.y, min.z);
	m_Vert[5] = CVector(max.x, max.y, min.z);
	m_Vert[6] = CVector(max.x, max.y, max.z);
	m_Vert[7] = CVector(min.x, max.y, max.z);

}

CVector CBoundingBox::GetVertex(unsigned int x, unsigned int y, unsigned int z)
{
	float v_x = -1, v_y = -1, v_z = -1;

	(x == 0)?v_x = m_Min.x:v_x = m_Max.x;
	(y == 0)?v_y = m_Min.y:v_y = m_Max.y;
	(z == 0)?v_z = m_Min.z:v_z = m_Max.z;


	return CVector(v_x, v_y, v_z);
}

void CBoundingBox::Draw()
{
	if(!Visible())
		return;

	glPushMatrix();

	if(m_pMdlMatrix !=NULL)
		glMultMatrixf(m_pMdlMatrix->m_matrix);

	ApplyMaterial(NULL, TEX_COLOR, m_Color.m_rgba);

	// Front
	glBegin(m_iRenderMode);
	glVertex3fv(m_Vert[0].m_pos);
	glVertex3fv(m_Vert[1].m_pos);
	glVertex3fv(m_Vert[2].m_pos);
	glVertex3fv(m_Vert[3].m_pos);
	glEnd();

	// Back
	glBegin(m_iRenderMode);
	glVertex3fv(m_Vert[4].m_pos);
	glVertex3fv(m_Vert[5].m_pos);
	glVertex3fv(m_Vert[6].m_pos);
	glVertex3fv(m_Vert[7].m_pos);
	glEnd();

	// Left
	glBegin(m_iRenderMode);
	glVertex3fv(m_Vert[0].m_pos);
	glVertex3fv(m_Vert[3].m_pos);
	glVertex3fv(m_Vert[7].m_pos);
	glVertex3fv(m_Vert[4].m_pos);
	glEnd();

	// Right
	glBegin(m_iRenderMode);
	glVertex3fv(m_Vert[1].m_pos);
	glVertex3fv(m_Vert[2].m_pos);
	glVertex3fv(m_Vert[6].m_pos);
	glVertex3fv(m_Vert[5].m_pos);
	glEnd();

	// Top
	glBegin(m_iRenderMode);
	glVertex3fv(m_Vert[3].m_pos);
	glVertex3fv(m_Vert[2].m_pos);
	glVertex3fv(m_Vert[6].m_pos);
	glVertex3fv(m_Vert[7].m_pos);
	glEnd();

	// Bottom
	glBegin(m_iRenderMode);
	glVertex3fv(m_Vert[0].m_pos);
	glVertex3fv(m_Vert[1].m_pos);
	glVertex3fv(m_Vert[5].m_pos);
	glVertex3fv(m_Vert[4].m_pos);
	glEnd();

	glPopMatrix();
}


void CBoundingBox::Evaluate(CVector *vertices, int count, CMatrix *mdl_matrix)
{
	if(vertices == NULL || count <=0 || mdl_matrix == NULL)
		return;

	CVector min = CVector(0, 0, 0), max = min;
		
	for(int i = 0; i < count; i++)
    {
	   if(vertices[i].x < min.x) min.x = vertices[i].x;
	   if(vertices[i].x > max.x) max.x = vertices[i].x;

	   if(vertices[i].y < min.y) min.y = vertices[i].y;
	   if(vertices[i].y > max.y) max.y = vertices[i].y;

	   if(vertices[i].z < min.z) min.z = vertices[i].z;
	   if(vertices[i].z > max.z) max.z = vertices[i].z;
    }
	
	m_pMdlMatrix = mdl_matrix;
	Calculate(min, max);
}

void CBoundingBox::Evaluate(tBoneVertex *vertices, int count, CMatrix *mdl_matrix)
{
	if(vertices == NULL || count <=0 || mdl_matrix == NULL)
		return;

	tBoneVertex min, max;
	min = CVector(10000,10000,10000);
	max = CVector(-10000,-10000,-10000);
		
	for(int i = 0; i < count; i++)
    {
	   if(vertices[i].m_Pos[0] < min.m_Pos[0]) min.m_Pos[0] = vertices[i].m_Pos[0];
	   if(vertices[i].m_Pos[0] > max.m_Pos[0]) max.m_Pos[0] = vertices[i].m_Pos[0];

	   if(vertices[i].m_Pos[1] < min.m_Pos[1]) min.m_Pos[1] = vertices[i].m_Pos[1];
	   if(vertices[i].m_Pos[1] > max.m_Pos[1]) max.m_Pos[1] = vertices[i].m_Pos[1];

	   if(vertices[i].m_Pos[2] < min.m_Pos[2]) min.m_Pos[2] = vertices[i].m_Pos[2];
	   if(vertices[i].m_Pos[2] > max.m_Pos[2]) max.m_Pos[2] = vertices[i].m_Pos[2];
    }
	
	m_pMdlMatrix = mdl_matrix;
	Calculate(CVector(min.m_Pos[0], min.m_Pos[1], min.m_Pos[2]), CVector(max.m_Pos[0], max.m_Pos[1], max.m_Pos[2]));
}

void CBoundingBox::Evaluate(tVERTEX3f *vertices, int count, CMatrix *mdl_matrix)
{
	if(vertices == NULL || count <=0 || mdl_matrix == NULL)
		return;

	tVERTEX3f min = tVERTEX3f(0, 0, 0), max = min;
		
	for(int i = 0; i < count; i++)
    {
	   if(vertices[i].x < min.x) min.x = vertices[i].x;
	   if(vertices[i].x > max.x) max.x = vertices[i].x;

	   if(vertices[i].y < min.y) min.y = vertices[i].y;
	   if(vertices[i].y > max.y) max.y = vertices[i].y;

	   if(vertices[i].z < min.z) min.z = vertices[i].z;
	   if(vertices[i].z > max.z) max.z = vertices[i].z;
    }

	m_pMdlMatrix = mdl_matrix;
	Calculate(CVector(min.x, min.y, min.z), CVector(max.x, max.y, max.z));
}

bool CBoundingBox::PointInBox(CVector vec)
{
	return (vec.x > m_Min.x && vec.x < m_Max.x) & (vec.y > m_Min.y && vec.y < m_Max.y) & (vec.z > m_Min.z && vec.z < m_Max.z);
}