// Model.cpp: implementation of the CModel class.
//
//////////////////////////////////////////////////////////////////////
#include "Model.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CModel::CModel()
{
	m_iNumMaterials = 0;
	m_iNumVertices = 0;
	m_iNumTriangles = 0;
	m_iNumMeshes = 0;
	
	m_pMaterials = NULL;
	m_pVertices = NULL;
	m_pTriangles = NULL;
	m_pMeshes = NULL;
		
	strcpy(m_strName, "GModel");
	m_iListID = 0;
	m_eRM = RENDER_MODE_SIMPLE;
	m_eMaterialRender = MAT;
	m_ID = 0;
	m_iVertexRender = GL_TRIANGLES;
	m_iNumJoints = 0;
	m_pJoints = NULL;
	m_bLooping = true;

	Restart();
}

void CModel::Destroy_Model()
{
	for (int i = 0; i < m_iNumMeshes; i++ )
		SAFEDEL(m_pMeshes[i].m_pTriangleIndices);
		
//	for ( i = 0; i < m_numMaterials; i++ )
//		m_pMaterials[i].Destroy();

	m_iNumMeshes = 0;
	SAFEDEL(m_pMeshes);
	m_pMeshes = NULL;
	
	m_iNumMaterials = 0;
	delete[] m_pMaterials;
	m_pMaterials = NULL;
	
	m_iNumTriangles = 0;
	SAFEDEL(m_pTriangles);
	m_pTriangles = NULL;
	
	m_iNumVertices = 0;
	SAFEDEL(m_pVertices);
	m_pVertices = NULL;	

	for ( i = 0; i < m_iNumJoints; i++ )
	{
		SAFEDEL(m_pJoints[i].m_pRotationKeyframes);
		SAFEDEL(m_pJoints[i].m_pTranslationKeyframes);
	}

	m_iNumJoints = 0;
	if ( m_pJoints != NULL )
	{
		SAFEDEL(m_pJoints);
		m_pJoints = NULL;
	}
}

// Precalculate normals for all triangles
void CModel::calculateNormals()
{
	for ( int i = 0; i < m_iNumMeshes; i++ )
	{
		for ( int j = 0; j < m_pMeshes[i].m_numTriangles; j++ )
		{
			int triangleIndex = m_pMeshes[i].m_pTriangleIndices[j];
			tTriangle* pTri = &m_pTriangles[triangleIndex];
						
			CVector vertex, pt[3];
				for (int t=0; t<3; t++)
					pt[t] = CVector(m_pVertices[(unsigned int)pTri->m_vertexIndices[t]].m_Pos[0], m_pVertices[(unsigned int)pTri->m_vertexIndices[t]].m_Pos[1], m_pVertices[(unsigned int)pTri->m_vertexIndices[t]].m_Pos[2]);

				vertex.SetNormal(pt[0], pt[1], pt[2]);
			for (int l=0; l<3; l++)
			{
				pTri->m_vertexNormals[l][0] = vertex.x;
				pTri->m_vertexNormals[l][1] = vertex.y;
				pTri->m_vertexNormals[l][2] = vertex.z;
			}
		}
	}

}
// ----------------------------------------------------------------
// Draws model in general way 
// ----------------------------------------------------------------
void CModel::Draw()
{
	if(m_iNumJoints > 0)
		Animate();

	switch(m_eRM)
	{
		case RENDER_MODE_SIMPLE:
		{
			// Draw by group
	//		glMultMatrixf(m_ViewMatrix.m_matrix);
					
			int face_count = 0;
			glPushMatrix();
			glMultMatrixf(m_ViewMatrix.m_matrix);
			for ( int i = 0; i < m_iNumMeshes; i++ )
			{
				int materialIndex = m_pMeshes[i].m_materialIndex;
					
				if(materialIndex >= 0)
					ApplyMaterial(&m_pMaterials[materialIndex], m_eMaterialRender);
				
				//	ApplyMaterial(&m_pMaterials[materialIndex], MAT_TEX);
			
				glLoadName(m_ID);		// Load Our Object's ID For Picking
				
				glBegin( m_iVertexRender );
				
					for ( int j = 0; j < m_pMeshes[i].m_numTriangles; j++ )
					{
						int triangleIndex = m_pMeshes[i].m_pTriangleIndices[j];
						tTriangle* pTri = &m_pTriangles[triangleIndex];
						
						for ( int k = 0; k < 3; k++ )
						{
							int index = pTri->m_vertexIndices[k];
							
							// { NEW }
							if ( m_pVertices[index].m_boneID == -1 )
							{
								// same as before
								glTexCoord2f( pTri->m_s[k], pTri->m_t[k] );
								glNormal3fv( pTri->m_vertexNormals[k] );
								glVertex3fv( m_pVertices[index].m_Pos );
							}
							else
							{
									// rotate according to transformation matrix
								CMatrix& final = m_pJoints[m_pVertices[index].m_boneID].m_final;

								glTexCoord2f( pTri->m_s[k], pTri->m_t[k] );

								CVector newNormal( pTri->m_vertexNormals[k][0], pTri->m_vertexNormals[k][1], pTri->m_vertexNormals[k][2], 1 );
								newNormal.Transform3( &final );
								newNormal.Normalize();
								glNormal3fv( newNormal.m_pos);

								CVector newVertex( m_pVertices[index].m_Pos[0], m_pVertices[index].m_Pos[1], m_pVertices[index].m_Pos[2] );
								newVertex.Transform( &final );
								glVertex3fv( newVertex.m_pos);
							}
						}
					}
				
				glEnd();
			}
			glPopMatrix();

			break;
		}

		case RENDER_MODE_DLIST:
		{
			if(m_iListID == 0)
			{
				m_iListID = glGenLists(1);
				glNewList (m_iListID, GL_COMPILE);
			
				int face_count = 0;
				for ( int i = 0; i < m_iNumMeshes; i++ )
				{
					int materialIndex = m_pMeshes[i].m_materialIndex;
						
					
					if(materialIndex >=0)
						ApplyMaterial(&m_pMaterials[materialIndex], m_eMaterialRender);

					//	ApplyMaterial(&m_pMaterials[materialIndex], MAT_TEX);
				
					glLoadName(m_ID);		// Load Our Object's ID For Picking
					
					glBegin( m_iVertexRender );
					
						for ( int j = 0; j < m_pMeshes[i].m_numTriangles; j++ )
						{
							int triangleIndex = m_pMeshes[i].m_pTriangleIndices[j];
							tTriangle* pTri = &m_pTriangles[triangleIndex];
							
							for ( int k = 0; k < 3; k++ )
							{
								int index = pTri->m_vertexIndices[k];
								
								glNormal3fv( pTri->m_vertexNormals[k] );
								glTexCoord2f( pTri->m_s[k], pTri->m_t[k] );
								glVertex3fv(  m_pVertices[index].m_Pos);
							}
						}
					glEnd();
				}				
				glEndList();
			}
			else
			{
				glPushMatrix();
				glMultMatrixf(m_ViewMatrix.m_matrix);
				glCallList(m_iListID);
				glPopMatrix();
			}

			break;
		}

		case RENDER_MODE_VARRAY:
		{
			
			break;
		}
 	}

	m_BBox.Draw();
}

// NEW
void CModel::SetJointKeyframe( int jointIndex, int keyframeIndex, float time, float *parameter, bool isRotation )
{
	if(!( m_iNumJoints > jointIndex))
		return;

	tKeyframe& keyframe = isRotation ? m_pJoints[jointIndex].m_pRotationKeyframes[keyframeIndex] :
		m_pJoints[jointIndex].m_pTranslationKeyframes[keyframeIndex];

	keyframe.m_jointIndex = jointIndex;
	keyframe.m_time = time;
	memcpy( keyframe.m_parameter, parameter, sizeof( float )*3 );
}

void CModel::SetupJoints()
{
	int i;
	for ( i = 0; i < m_iNumJoints; i++ )
	{
		tJoint& joint = m_pJoints[i];

		joint.m_relative.RotateRadians( joint.m_localRotation[0], joint.m_localRotation[1], joint.m_localRotation[2] );
		joint.m_relative.SetTranslation( joint.m_localTranslation );
		if ( joint.m_parent != -1 )
		{
			joint.m_absolute = m_pJoints[joint.m_parent].m_absolute;
			joint.m_absolute.Multiply(&joint.m_relative);
		}
		else
			joint.m_absolute = joint.m_relative;
	}

	for ( i = 0; i < m_iNumVertices; i++ )
	{
		tBoneVertex& vertex = m_pVertices[i];

		if ( vertex.m_boneID != -1 )
		{
			CMatrix& matrix = m_pJoints[vertex.m_boneID].m_absolute;

			matrix.InverseTranslateVect( vertex.m_Pos );
			matrix.InverseRotateVect( vertex.m_Pos );
		}
	}

	for ( i = 0; i < m_iNumTriangles; i++ ) {
		tTriangle& triangle = m_pTriangles[i];
		for ( int j = 0; j < 3; j++ ) {
			tBoneVertex& vertex = m_pVertices[(const int)triangle.m_vertexIndices[j]];
			if ( vertex.m_boneID != -1 ) {
				CMatrix& matrix = m_pJoints[vertex.m_boneID].m_absolute;
				matrix.InverseRotateVect( triangle.m_vertexNormals[j] );
			}
		}
	}
}

void CModel::Restart()
{
	for ( int i = 0; i < m_iNumJoints; i++ )
	{
		m_pJoints[i].m_currentRotationKeyframe = m_pJoints[i].m_currentTranslationKeyframe = 0;
		m_pJoints[i].m_final = m_pJoints[i].m_absolute;
	}

	m_Timer.reset();
}

void CModel::Animate()
{
	double time = m_Timer.getTime();

	if ( time > m_fTotalTime )
	{
		if ( m_bLooping )
		{
			Restart();
			time = 0;
		}
		else
			time = m_fTotalTime;
	} 

	for ( int i = 0; i < m_iNumJoints; i++ )
	{
		float transVec[3];
		CMatrix transform;
		int frame;
		tJoint *pJoint = &m_pJoints[i];

		if ( pJoint->m_numRotationKeyframes == 0 && pJoint->m_numTranslationKeyframes == 0 )
		{
			pJoint->m_final = pJoint->m_absolute;
			continue;
		}

		frame = pJoint->m_currentTranslationKeyframe;
		while ( frame < pJoint->m_numTranslationKeyframes && pJoint->m_pTranslationKeyframes[frame].m_time < time )
		{
			frame++;
		}
		pJoint->m_currentTranslationKeyframe = frame;

		if ( frame == 0 )
			memcpy( transVec, pJoint->m_pTranslationKeyframes[0].m_parameter, sizeof ( float )*3 );
		else if ( frame == pJoint->m_numTranslationKeyframes )
			memcpy( transVec, pJoint->m_pTranslationKeyframes[frame-1].m_parameter, sizeof ( float )*3 );
		else
		{
			if(!(frame > 0 && frame < pJoint->m_numTranslationKeyframes ))
				return;

			tKeyframe curFrame = pJoint->m_pTranslationKeyframes[frame];
			tKeyframe prevFrame = pJoint->m_pTranslationKeyframes[frame-1];

			float timeDelta = curFrame.m_time-prevFrame.m_time;
			float interpValue = ( float )(( time-prevFrame.m_time )/timeDelta );

			transVec[0] = prevFrame.m_parameter[0]+( curFrame.m_parameter[0]-prevFrame.m_parameter[0] )*interpValue;
			transVec[1] = prevFrame.m_parameter[1]+( curFrame.m_parameter[1]-prevFrame.m_parameter[1] )*interpValue;
			transVec[2] = prevFrame.m_parameter[2]+( curFrame.m_parameter[2]-prevFrame.m_parameter[2] )*interpValue; 
		}

		frame = pJoint->m_currentRotationKeyframe;
		while ( frame < pJoint->m_numRotationKeyframes && pJoint->m_pRotationKeyframes[frame].m_time < time )
		{
			frame++;
		}
		pJoint->m_currentRotationKeyframe = frame;

		if ( frame == 0 )
			transform.RotateRadians( pJoint->m_pRotationKeyframes[0].m_parameter[0], pJoint->m_pRotationKeyframes[0].m_parameter[1], pJoint->m_pRotationKeyframes[0].m_parameter[2] );
		else if ( frame == pJoint->m_numRotationKeyframes )
			transform.RotateRadians( pJoint->m_pRotationKeyframes[frame-1].m_parameter[0], pJoint->m_pRotationKeyframes[frame-1].m_parameter[1], pJoint->m_pRotationKeyframes[frame-1].m_parameter[2] );
		else
		{
			if(!( frame > 0 && frame < pJoint->m_numRotationKeyframes ))
				return;

			tKeyframe curFrame = pJoint->m_pRotationKeyframes[frame];
			tKeyframe prevFrame = pJoint->m_pRotationKeyframes[frame-1];
			
			float timeDelta = curFrame.m_time-prevFrame.m_time;
			float interpValue = ( float )(( time-prevFrame.m_time )/timeDelta );

			if(!( interpValue >= 0 && interpValue <= 1 ))
				return;

#if 0
			CQuaternion qPrev( prevFrame.m_parameter );
			CQuaternion qCur( curFrame.m_parameter );
			CQuaternion qFinal( qPrev, qCur, interpValue );
			MatrixToQuaternion(&qFinal, &transform);
#else
			float rotVec[3];

			rotVec[0] = prevFrame.m_parameter[0]+( curFrame.m_parameter[0]-prevFrame.m_parameter[0] )*interpValue;
			rotVec[1] = prevFrame.m_parameter[1]+( curFrame.m_parameter[1]-prevFrame.m_parameter[1] )*interpValue;
			rotVec[2] = prevFrame.m_parameter[2]+( curFrame.m_parameter[2]-prevFrame.m_parameter[2] )*interpValue;

			transform.RotateRadians( rotVec[0], rotVec[1], rotVec[2] );
#endif
		}

		transform.SetTranslation( CVector(transVec[0], transVec[1], transVec[2], 1) );
		CMatrix relativeFinal( pJoint->m_relative );
		relativeFinal.Multiply( &transform );

		if ( pJoint->m_parent == -1 )
			pJoint->m_final = relativeFinal;
		else
		{
			pJoint->m_final = m_pJoints[pJoint->m_parent].m_final;
			pJoint->m_final.Multiply( &relativeFinal );
		}
	}
}
// { end NEW }