// OctreeBBox.cpp: implementation of the COctreeBBox class.
//
//////////////////////////////////////////////////////////////////////

#include "OctreeBBox.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

COctreeBBox::COctreeBBox()
{
	for (int i=0; i<8; i++)
		m_Node[i] = NULL;

	m_iDivLvl = 0;
}

COctreeBBox::~COctreeBBox()
{
	Destroy();
}

void COctreeBBox::Destroy()
{
	for (int i=0; i<8; i++)
	{
		if(m_Node[i] !=NULL)
			delete m_Node[i];
	}
}

void COctreeBBox::Draw()
{
	if(Visible())
		CBoundingBox::Draw();

	for (int i=0; i<8; i++)
	{
		if(m_Node[i] !=NULL)
		{
			m_Node[i]->m_pMdlMatrix = m_pMdlMatrix;
			m_Node[i]->Draw();
		}
	}
}

void COctreeBBox::Subdivide(unsigned int lvl)
{
	// An octree box consists of 8 (hence octree) sub-boxes
	// Start sub-dividing the box by giving minimum and maximum points to appropriate sub-boxes

	if(lvl == 0)
		return;

	m_iDivLvl = lvl;

	for (int i=0; i<8; i++)
		m_Node[i] = new COctreeBBox;

	CVector min_orig = GetVertex(0,0,0);
	CVector max = GetVertex(1,1,1);
	CVector min = CVector(0,0,0);
	max = max - min_orig;


	// Front bottom left
	GetNode(0)->Calculate(min_orig + min, min_orig + max/2);
	
	// Front bottom right
	GetNode(1)->Calculate(min_orig + CVector(max.x/2, 0, 0), min_orig + CVector(max.x, max.y/2, max.z/2));

	// Front top right
	GetNode(2)->Calculate(min_orig + CVector(max.x/2, 0, max.z/2), min_orig + CVector(max.x, max.y/2, max.z));

	// Front top left
	GetNode(3)->Calculate(min_orig + CVector(0, 0, max.z/2), min_orig + CVector(max.x/2, max.y/2, max.z));
	
	// Back bottom left
	GetNode(4)->Calculate(min_orig + CVector(0, max.y/2, 0), min_orig + CVector(max.x/2, max.y, max.z/2));

	// Back bottom right
	GetNode(5)->Calculate(min_orig + CVector(max.x/2, max.y/2, 0), min_orig + CVector(max.x, max.y, max.z/2));

	// Back top right
	GetNode(6)->Calculate(min_orig + max/2, min_orig + max);

	// Back top left
	GetNode(7)->Calculate(min_orig + CVector(0, max.y/2, max.z/2), min_orig + CVector(max.x/2, max.y, max.z));

	lvl--;
	if(lvl > 0)
	{
		for (int i=0; i<8; i++)
			GetNode(i)->Subdivide(lvl);
	}
}

void COctreeBBox::ClearEmpty(CVector *vertices, int count)
{
	for (int j=0; j<8; j++)
	{
		for (int i=0; i<count; i++)
		{
			if(!m_Node[j]->PointInBox(vertices[i]))
			{
				delete m_Node[j];
				m_Node[j] = NULL;
			}

		}

		m_Node[j]->ClearEmpty(vertices, count);
	}
}

void COctreeBBox::ClearEmpty(tBoneVertex *vertices, int count)
{	
	for (int i=0; i<count; i++)
	{
		for (int j=0; j<8; j++)
		{
			if(m_Node[j] !=NULL && !m_Node[j]->PointInBox(CVector(vertices[i].m_Pos[0], vertices[i].m_Pos[1], vertices[i].m_Pos[2]) ))
			{
				delete m_Node[j];
				m_Node[j] = NULL;
			}
			else if(m_Node[j] !=NULL)
			{
				m_Node[j]->ClearEmpty(vertices, count);
				m_Node[j]->CheckNodesVisibility();
			}
		}
	}
}

void COctreeBBox::ClearEmpty(tVERTEX3f *vertices, int count)
{
	for (int j=0; j<8; j++)
	{
		for (int i=0; i<count; i++)
		{
			if(m_Node[j]->PointInBox(CVector(vertices[i].x, vertices[i].y, vertices[i].z)))
			{
				m_Node[j]->Show();
				break;
			}
			else
				m_Node[j]->Hide();
		}

		m_Node[j]->ClearEmpty(vertices, count);
	}
}

void COctreeBBox::CheckNodesVisibility()
{
	bool vis = false;

	for (int j=0; j<8; j++)
	{
		if(m_Node[j] !=NULL && m_Node[j]->Visible())	
		{
			vis = true;
			m_Node[j]->CheckNodesVisibility();
		}
	}

	if(vis)
		Hide();
	else Show();
}